""" $lic$
Copyright (c) 2016-2017, Mingyu Gao
All rights reserved.

This program is free software: you can redistribute it and/or modify it under
the terms of the Modified BSD-3 License as published by the Open Source
Initiative.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the BSD-3 License for more details.

You should have received a copy of the Modified BSD-3 License along with this
program. If not, see <https://opensource.org/licenses/BSD-3-Clause>.
"""

from collections import namedtuple

# Alphabetical order.
_TIMING_PARAM_LIST = [
    'RAS',
    'REFI',
    'RFC',
    'RP',
    'RRD',
    ]

'''
Define timing parameters in unit of cycles.
'''
Timing = namedtuple('Timing', _TIMING_PARAM_LIST)

